package cn.x47.handle;


import cn.x47.model.RIPEntry;
import cn.x47.model.RIPPacket;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.socket.DatagramPacket;
import io.netty.handler.codec.MessageToMessageDecoder;

import java.net.InetAddress;
import java.util.ArrayList;
import java.util.List;

/**
 * 解码器
 */
public class RIPPacketDecoder extends MessageToMessageDecoder<DatagramPacket> {

    @Override
    protected void decode(ChannelHandlerContext ctx, DatagramPacket packet, List<Object> out) throws Exception {
        ByteBuf buf = packet.content();
        RIPPacket ripPacket = new RIPPacket();
        ripPacket.setCommand(buf.readByte());
        ripPacket.setVersion(buf.readByte());
        ripPacket.setUnused(buf.readShort());

        List<RIPEntry> entries = new ArrayList<>();
        while (buf.readableBytes() >= 20) {
            RIPEntry entry = new RIPEntry();
            entry.setAddressFamily((short) buf.readUnsignedShort());

            if (ripPacket.getVersion() == 2) {
                entry.setRouteTag((short) buf.readUnsignedShort());
            } else {
                entry.setRouteTag((short) 0);
            }

            byte[] ipBytes = new byte[4];
            buf.readBytes(ipBytes);
            entry.setIpAddress(InetAddress.getByAddress(ipBytes));

            if (ripPacket.getVersion() == 2) {
                buf.readBytes(ipBytes);
                entry.setSubnetMask(InetAddress.getByAddress(ipBytes));
                buf.readBytes(ipBytes);
                entry.setNextHop(InetAddress.getByAddress(ipBytes));
            } else {
                // 对于 RIP v1，子网掩码和下一跳需要推断或设为默认值
                entry.setSubnetMask(InetAddress.getByName("255.255.255.0")); // 示例，实际应根据 IP 类推断
                entry.setNextHop(packet.sender().getAddress());
            }

            long metric = buf.readUnsignedInt();
            entry.setMetric(metric);

            entries.add(entry);
        }
        ripPacket.setEntries(entries);
        ripPacket.setSenderAddress(packet.sender()); // 设置发送者地址
        out.add(ripPacket);
    }

}
